/**********************************************
	COMS W4115 PLT - 2014 Spring Term Project
	HL-HDL compiler: parser.mly
		Name:	Woon Lee
		UNI:	wgl2107
***********************************************/

%{ open Ast %}

%token EOF INSIG OUTSIG IOSIG CONST SIGNAL AND OR INV BUF LAT MUX LBRACKET RBRACKET
%token FUNC IF THEN ELSE MAIN COMMA LPAREN RPAREN LBRACE RBRACE SEMI COLON ASSIGN IASSIGN
%token EQ NEQ
%token <string> ID BNUM HNUM
%token <int> INUM

%nonassoc NOELSE
%nonassoc ELSE
%right ASSIGN
%left AND OR EQ NEQ

%start program
%type <Ast.program> program

%%

program:
	/* nothing */	{ [], [] }
	| program vdecl { ($2 :: fst $1), snd $1 }
	| program fdecl { fst $1, ($2 :: snd $1) }
	
fdecl:
	fn_type ID LPAREN formals_opt RPAREN LBRACE vdecl_list stmt_list RBRACE
								{ { func_type	= $1;
									func_name	= $2;
									formals		= $4;
									locals		= List.rev $7;
									body		= List.rev $8 }
								}
fn_type:
	  FUNC	{ User }
	| MAIN	{ Main }

formals_opt:
	/* nothing */		{ [] }
	| formal_list		{ List.rev $1 }
	
formal_list:
	formal_decl				{ [$1] }
	| formal_list COMMA formal_decl	{ $3 :: $1 }
	
formal_decl:
	formaltype ID bus_opt	{ { formal_type = $1; formal_name = $2; formal_range = $3 } }

formaltype:
	  INSIG		{ Insig }
	| OUTSIG	{ Outsig }
	| IOSIG		{ IOsig }

bus_opt:
	/* nothing */					{ (-1, -1) }
	| LBRACKET INUM COLON INUM RBRACKET	{ ($2, $4) }

vdecl_list:
	/* nothing */			{ [] }
	| vdecl_list vdecl		{ $2 :: $1 }

vdecl:
	var_decl SEMI			{ $1 }

var_decl:
	vartype ID bus_opt init_value			{ { var_type = $1; var_name = $2; var_range = $3; var_value = $4 } }

vartype:
	  CONST		{ Const }
	| SIGNAL	{ Signal }

init_value:
	/* nothing */		{ "none" }
	| IASSIGN INUM		{ string_of_int $2 }	
	| IASSIGN BNUM		{ $2 }
	| IASSIGN HNUM		{ $2 }
	
stmt_list:
	/* nothing */			{ [] }
	| stmt_list stmt		{ $2 :: $1 }
	
stmt:
	expr SEMI										{ Expr($1) }
	| LBRACE stmt_list RBRACE						{ Block(List.rev $2) }
	| id_opt IF LPAREN expr RPAREN THEN stmt %prec NOELSE	{ If ($1, $4, $7, Block([])) }
	| id_opt IF LPAREN expr RPAREN THEN stmt ELSE stmt		{ If ($1, $4, $7, $9) }
	| expr		{ StError ($1) }
	
id_opt:
	/* nothing */		{ "none" }
	| ID COLON			{ $1 }
	
expr:
	  ID							{ Id($1) }
	| INUM							{ Lit_int($1) }
	| HNUM							{ Lit_hex($1) }
	| BNUM							{ Lit_bin($1) }
	| ID LBRACKET INUM RBRACKET		{ Single($1, $3) }
	| ID LBRACKET INUM COLON INUM RBRACKET		{ Range($1, $3, $5) }
	| expr AND expr					{ Binop($1, And, $3) }
	| expr OR expr					{ Binop($1, Or, $3) }
	| expr EQ expr					{ Binop($1, Equal, $3) }
	| expr NEQ expr					{ Binop($1, Nequal, $3) }
	| asgn_opt ASSIGN expr			{ Assign($1, $3) }
	| ID COLON ID LPAREN actuals_opt RPAREN	{ Call($1, $3, $5) }
	| INV LPAREN actuals_opt RPAREN	{ Bcall("", Inv, $3) }
	| BUF LPAREN actuals_opt RPAREN	{ Bcall("", Buf, $3) }
	| id_opt LAT LPAREN actuals_opt RPAREN	{ Bcall($1, Lat, $4) }
	| id_opt MUX LPAREN actuals_opt RPAREN	{ Bcall($1, Mux, $4) }
	| LPAREN expr RPAREN			{ Braces($2) }
	
asgn_opt:
	ID				{ Id($1) }
	| ID LBRACKET INUM RBRACKET		{ Single($1, $3) }
	| ID LBRACKET INUM COLON INUM RBRACKET		{ Range($1, $3, $5) }
	
actuals_opt:
	/* nothing */	{ [] }
	| actuals_list	{ List.rev $1 }
	
actuals_list:
	expr						{ [$1] }
	| actuals_list COMMA expr	{ $3 :: $1 }