----------------------------------------------------
--  COMS W4115 PLT - 2014 Spring Term Project
--              HL-HDL compiler
--             Woon Lee (wgl2107)
----------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use work.globals.all;
use work.lib_comp.all;



ENTITY test6_top_tb is

end ENTITY test6_top_tb;

ARCHITECTURE behavioral of test6_top_tb is

procedure RegRd (
		signal cs_n		: 	out		std_logic;
		signal read_n 	: 	out		std_logic;
		signal write_n 	: 	out		std_logic;
		signal reg_addr	:	in		std_logic_vector(3 downto 0);
		signal addr_out	: 	out		std_logic_vector(3 downto 0)
) is
		
begin
	cs_n <= '1';
	read_n <= '1';
	write_n <= '1';
	wait for 30 ns;
		
	addr_out <= reg_addr;
	wait for 5 ns;
	cs_n <= '0';
	wait for 5 ns;
	read_n <= '0';		-- read access 
	wait for 60 ns;
	read_n <= '1';
	wait for 5 ns;
	cs_n <= '1';
	wait for 20 ns;
end procedure;

procedure RegWr (
		signal cs_n		: 	out		std_logic;
		signal read_n 	: 	out		std_logic;
		signal write_n 	: 	out		std_logic;
		signal reg_addr	:	in		std_logic_vector(3 downto 0);
		signal addr_out	: 	out		std_logic_vector(3 downto 0);
		signal data_in	:	in		std_logic_vector(7 downto 0);
		signal data_out	:	out		std_logic_vector(7 downto 0)
) is
		
begin
	cs_n <= '1';
	read_n <= '1';
	write_n <= '1';
	wait for 30 ns;
		
	addr_out <= reg_addr;
	wait for 5 ns;
	cs_n <= '0';
	data_out <= data_in;
	wait for 5 ns;
	write_n <= '0';		-- read access 
	wait for 60 ns;
	write_n <= '1';
	wait for 5 ns;
	cs_n <= '1';
	data_out <= (others => 'Z');
	wait for 20 ns;
end procedure;

COMPONENT test6_top is
port (
	clk : in  std_logic;
	rst : in  std_logic;
	csn : in  std_logic;
	oen : in  std_logic;
	wrn : in  std_logic;
	addr : in std_logic_vector(3 downto 0);
	data : inout std_logic_vector(7 downto 0)
);
end COMPONENT;

	signal clk :  std_logic;
	signal rst :  std_logic;
	signal csn :  std_logic;
	signal oen :  std_logic;
	signal wrn :  std_logic;
	signal addr : std_logic_vector(3 downto 0);
	signal data : std_logic_vector(7 downto 0);
	signal register_addr : std_logic_vector(3 downto 0);
	signal wr_value : std_logic_vector(7 downto 0);
	signal sim_done : boolean := false;
	
begin

inst_test6_top : test6_top
	port map(
		clk => clk,
		rst => rst,
		csn => csn,
		oen => oen,
		wrn => wrn,
		addr => addr,
		data => data
	);

clk_proc : process 
	begin
		if (not sim_done) then
			clk <= '0';
			wait for 10 ns;
			clk <= '1';
			wait for 10 ns;
		else
			report "sim_done";
			wait; --wait forever
		end if;
	end process clk_proc;

reg_rd_wr_seq : process
	begin
		data <= (others => 'Z');
		wait for 30 ns;
		-- check version and revision
		register_addr <= x"0"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"1"; RegRd(csn, oen, wrn, register_addr, addr);
		-- write all 14 registers with unique values, respectively	
		register_addr <= x"2"; wr_value <= x"22"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		register_addr <= x"3"; wr_value <= x"33"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		register_addr <= x"4"; wr_value <= x"44"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		register_addr <= x"5"; wr_value <= x"55"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		register_addr <= x"6"; wr_value <= x"66"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		register_addr <= x"7"; wr_value <= x"77"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		register_addr <= x"8"; wr_value <= x"88"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		register_addr <= x"9"; wr_value <= x"99"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		register_addr <= x"A"; wr_value <= x"AA"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		register_addr <= x"B"; wr_value <= x"BB"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		register_addr <= x"C"; wr_value <= x"CC"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		register_addr <= x"D"; wr_value <= x"DD"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		register_addr <= x"E"; wr_value <= x"EE"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		register_addr <= x"F"; wr_value <= x"FF"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		-- read back register values and verify
		register_addr <= x"0"; RegRd(csn, oen, wrn, register_addr, addr); 
		register_addr <= x"1"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"2"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"3"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"4"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"5"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"6"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"7"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"8"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"9"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"A"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"B"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"C"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"D"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"E"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"F"; RegRd(csn, oen, wrn, register_addr, addr);
		-- write different values to selected registers
		register_addr <= x"2"; wr_value <= x"E8"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		register_addr <= x"7"; wr_value <= x"5A"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		register_addr <= x"C"; wr_value <= x"A5"; RegWr(csn, oen, wrn, register_addr, addr, wr_value, data);
		-- read back and verify
		register_addr <= x"2"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"7"; RegRd(csn, oen, wrn, register_addr, addr);
		register_addr <= x"C"; RegRd(csn, oen, wrn, register_addr, addr);
				
		wait;
	end process;
		
sim_time : process
	begin
		rst <= '1';
		wait for 10 ns;
		rst <= '0';
		wait for 5000 ns;
		sim_done <= true;
		wait;
	end process;	

end;

